-- ############################################################################# 
-- # DC/DS Preflight Check - Lua application for JETI DC/DS transmitters  
-- #
-- # Copyright (c) 2016 - 2017, JETI model s.r.o.
-- # All rights reserved.
-- #
-- # Redistribution and use in source and binary forms, with or without
-- # modification, are permitted provided that the following conditions are met:
-- # 
-- # 1. Redistributions of source code must retain the above copyright notice, this
-- #    list of conditions and the following disclaimer.
-- # 2. Redistributions in binary form must reproduce the above copyright notice,
-- #    this list of conditions and the following disclaimer in the documentation
-- #    and/or other materials provided with the distribution.
-- # 
-- # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-- # ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-- # WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-- # DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
-- # ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-- # (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-- # LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-- # ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-- # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- # 
-- # The views and conclusions contained in the software and documentation are those
-- # of the authors and should not be interpreted as representing official policies,
-- # either expressed or implied, of the FreeBSD Project.                    
-- #                       
-- # V1.0 - Initial release
-- # V1.1 - Added Spanish and Italian language
-- # V1.2 - The readFile() function has been replaced by internal io.readall() (DC/DS FW V4.22)
-- # V1.2.1 - Modified by Geierwally: adapted with optional audiofiles for check and  full \ partial check 
-- # V1.2.2 - Bugfixed by Geierwally: 
-- #          1. question full or partial check opens if no switch is configured 
-- #          2. failure in partial check, wrong listindex set on optionExec- list
-- #############################################################################

--Configuration
local optionsValues={}
local cfgAudio
local cfgSwitch
--Local variables
local lang
local options={}    
local optionAudios={} -- modified by Geierwally: contains audio filenames for the preflight check slices
local optionExec={}   -- modified by Geierwally: contains information of preflight check slices executed in full - or partial check 
local appName="Preflight Check"
local selectboxes={} 
local checkboxes={}
local currentForm=0
local MAX_ITEMS = 20
local lastSwitchValue=true
local fullCheck=false -- modified by Geierwally: if value is true, a full preflight check is executed (done each 12 hours)
local currentTime=0	  -- modified by Geierwally: time stamp for calculation 12 hour full check time
local prevRow=0		  -- modified by Geierwally: previous row for detection new checkbox is selected  (play corresponding audio file of check slice)
local audioListIndex={}	  -- modified by Geierwally: contains corresponding index to audio files list 	
 
--------------------------------------------------------------------
-- Configure language settings
--------------------------------------------------------------------
local function setLanguage()
  -- Set language
  local lng=system.getLocale();
  local file = io.readall("Apps/Preflight/locale.jsn")
  local obj = json.decode(file)  
  if(obj) then
    lang = obj[lng] or obj[obj.default]
  end
end
--------------------------------------------------------------------
-- Setting forms
--------------------------------------------------------------------
local function checkButtons()
  if(currentForm==1) then
    form.setButton(3,":add", #optionsValues < MAX_ITEMS and ENABLED or DISABLED)
    form.setButton(4,":delete",#optionsValues>0 and ENABLED or DISABLED)
    form.setButton(1,":tools",ENABLED)
  else
    form.setButton(1,":play",ENABLED)
  end
end
--------------------------------------------------------------------

local function optionChanged()
  for k, v in ipairs(selectboxes) do 
    optionsValues[k]=form.getValue(v)
  end
  system.pSave("items",optionsValues)
end

local function audioChanged(value)
  cfgAudio=value
  system.pSave("audioOn",value)
end
local function switchChanged(value)
  cfgSwitch=value
  system.pSave("switch",value)
end
--------------------------------------------------------------------
local function initForm(formID)
  selectboxes={}
  currentForm=formID
  if(formID==1) then 
    for index=1,#optionsValues do
      form.addRow(2)
      form.addLabel({label=index..")",alignRight=true,width=40,font=FONT_BOLD})
      selectboxes[index]=form.addSelectbox(options,optionsValues[index],true,optionChanged,{width=260})
    end 
  else 
    form.addLabel({label=lang.cfgName,font=FONT_BOLD})
    -- Assigned audio file
    form.addRow(2)
    form.addLabel({label=lang.audio})
	form.addIntbox(cfgAudio,0,1,0,0,1,audioChanged)
    -- Assigned switch
    form.addRow(2)
    form.addLabel({label=lang.switch})
    form.addInputbox(cfgSwitch,false,switchChanged)
    
  end
  checkButtons()
end  
--------------------------------------------------------------------
local function keyPressed(key)
  if(currentForm==1) then
    if(key==KEY_3 and #optionsValues < MAX_ITEMS) then
      form.addRow(2)
      local vals = #optionsValues+1
      form.addLabel({label=(vals)..")",alignRight=true,width=40,font=FONT_BOLD})
      optionsValues[vals] = 0
      selectboxes[vals]=form.addSelectbox(options,0,true,optionChanged,{width=260})
      form.setFocusedRow(vals)
      --Save current selection
      optionChanged()
      checkButtons()
      return
    end
    local row = form.getFocusedRow()
    if(key==KEY_4 and row>0) then
      table.remove(optionsValues,row)
      table.remove(selectboxes,row) 
      optionChanged()
      form.reinit(1)
      return
    end 
    if(key==KEY_1) then
      form.reinit(2)
    end
  else
    if(key==KEY_1) then
      -- file playback
	  if(cfgAudio==1) then
	    -- print("Preflightcheck start")
		system.playFile("P_PlBack.wav",AUDIO_IMMEDIATE)
	  end	
    elseif(key==KEY_5 or key==KEY_ESC) then
      form.preventDefault()
      form.reinit(1)
    end
  end
end  
--------------------------------------------------------------------
-- Preflight check form
--------------------------------------------------------------------
local function clickedCallback(value)
  local row = form.getFocusedRow()
  form.setValue(checkboxes[row],true)
  local removeForm = true
  for index=1,#checkboxes do
    if(form.getValue(checkboxes[index]) == 0) then
      removeForm = false
    end
  end
  if (removeForm) then
  
  -- modified by Geierwally: if last preflight check was a full check, then safe current time as cycle time for next full check
	if(fullCheck==true)then
		fullCheck=false
		system.pSave("fullCheckTime",currentTime)
	end
    form.close()
  end
end
--------------------------------------------------------------------
local function initFormPrefl(formID)
  -- form.setButton(5,"",ENABLED)  
  local i=1
  local audioIndex=0
  audioListIndex={}
  
  checkboxes={}
  currentTime=system.getTime()
  local fullCheckTime = system.pLoad("fullCheckTime",0)
  
  -- modified by Geierwally: if time difference to last full preflight check is higher than 12 hours then start full preflight check 
  --                         otherwise only partial 
  if((currentTime-fullCheckTime > 43200)or (fullCheckTime == 0)) then
	fullCheck=true
  else
    fullCheck=false
	if(lastSwitchValue == true) then
	if(form.question(lang.qu_full_check,lang.qu_full_check_label,lang.qu_full_check_descr)==1)then
		--"Full check?","Full preflight check is started"
			fullCheck=true
		end	
	end	
  end
  
    
  for index=1,#optionsValues do
    -- modified by Geierwally:  if option is configured for full preflight check and timeout is not reached, don't fill corresponding 
 
	if(optionsValues[index] > 0)then
		if(optionExec[optionsValues[index]]=="full" and fullCheck==false) then -- bugfixed by Geierwally failure in partial check, wrong listindex set on optionExec- list
		-- modified by Geierwally: skip full checks if allready done
		else
		    if(audioIndex==0) then
				audioIndex=optionsValues[index] -- modified by Geierwally: set table marker for first audio file
				prevRow=i --modified by Geierwally reset previous row that the audio file is played for first focused row
			end
			audioListIndex[i]=optionsValues[index] -- modified by Geierwally: set index for audio file
			form.addRow(3)
			form.addLabel({label=i..")",alignRight=true,width=40,font=FONT_BOLD})
			form.addLabel({label=options[optionsValues[index]],width=220})
			checkboxes[i] = form.addCheckbox(false,clickedCallback)
			i=i+1
		end
	end
  end
  form.setButton(3,":down",ENABLED)
  -- Empty form - immediately close
  if(#checkboxes==0) then
    form.close()
  elseif(cfgAudio ==1) then
    -- file playback
    -- print("Preflightcheck start")
	system.playFile("P_PlBack.wav",AUDIO_QUEUE)
	
	 -- modified by Geierwally play audio file for first preflight check
	if(audioIndex > 0) then 
      -- print(optionAudios[audioIndex])
		system.playFile(optionAudios[audioIndex],AUDIO_QUEUE)
	end	
  end   
end 
--------------------------------------------------------------------
local function keyPressedPrefl(key)
  local row = form.getFocusedRow()
  local nextRow = row+1
  if(key==KEY_MENU or key==KEY_ESC) then
    form.preventDefault()
  elseif(key==KEY_3)then
    -- modified by Geierwally: next step pressed set actual box to checked and jump to next checkbox
    form.setValue(checkboxes[row],true)
	clickedCallback()
    if(nextRow <= #checkboxes) then
		form.setFocusedRow(nextRow)
	else
		form.setFocusedRow(1)
	end
  end
  -- modified by Geierwally if focus changes to other row, play corresponding audio file
  if(row ~= prevRow)then
    --rvtest
    if(cfgAudio==1)then
	--rvtest  print(optionAudios[audioListIndex[row]])
	  system.playFile(optionAudios[audioListIndex[row]],AUDIO_QUEUE)
	end  
    prevRow = row 
  end
end 

-------------------------------------------------------------------- 
-- Initialization
--------------------------------------------------------------------
-- Init function
local function init(code) 
  -- Load data
  local file = io.readall("Apps/Preflight/"..lang.data)
  
  -- modified by Geierwally separate from data file options , 
  --                        corresponding audio files and conditions for 
  --                        full or partial preflight check
  if(file) then
	local sa = string.find(file,"[%[]")
	local ea = string.find(file,"[%]]")
	local sb = string.find(file,"[%[]",ea+1)
	local eb = string.find(file,"[%]]",ea+1)
	local sc = string.find(file,"[%[]",eb+1)
	local ec = string.find(file,"[%]]",eb+1)
	options = json.decode(string.sub(file,sa,ea))
	optionAudios = json.decode(string.sub(file,sb,eb))
	optionExec =  json.decode(string.sub(file,sc,ec))
  end
 
  optionsValues = system.pLoad("items",{})
  cfgAudio = system.pLoad("audioOn","1")
  cfgSwitch = system.pLoad("switch")
  system.registerForm(1,MENU_ADVANCED,lang.appName,initForm,keyPressed,printForm);
  -- Show the form (only after model selection)
  if(#optionsValues > 0 and code==1) then
    system.registerForm(0,0,lang.appName,initFormPrefl,keyPressedPrefl);
  end 
end


  

--------------------------------------------------------------------
-- Loop function
local function loop() 
  local val
  if(cfgSwitch == nil)then -- bugfixed by Geierwally, don't open yes no box if no switch is configured
	val = 0
  else	
    val = system.getInputsVal(cfgSwitch)
  end	
  if(val and val>0 and not lastSwitchValue) then
    lastSwitchValue = true
    local frm=form.getActiveForm() 
    if(frm==0) then
      for index=1,#checkboxes do
        form.setValue(checkboxes[index],false)
      end
    elseif(#optionsValues > 0) then
      system.registerForm(0,0,lang.appName,initFormPrefl,keyPressedPrefl);
    end 
  elseif(val and val<=0) then 
    lastSwitchValue=false  
  end  
end
 

--------------------------------------------------------------------
setLanguage()
return { init=init, loop=loop, author="JETI model", version="1.2.2",name=lang.appName}